@@ -264,6 +264,8 @@ int SockOpen(const char *host, const char *service,
 {
     struct addrinfo *ai, req;
     int i, acterr = 0;
+    int ord;
+    char errbuf[8192] = "";
 
 #ifdef HAVE_SOCKETPAIR
     if (plugin)
@@ -285,10 +287,13 @@ int SockOpen(const char *host, const char *service,
 	return -1;
     }
 
+    /* NOTE a Linux bug here - getaddrinfo will happily return 127.0.0.1
+     * twice if no IPv6 is configured */
     i = -1;
-    for (ai = *ai0; ai; ai = ai->ai_next) {
-	char buf[80],pb[80];
-	int gnie;
+    for (ord = 0, ai = *ai0; ai; ord++, ai = ai->ai_next) {
+	char buf[256]; /* hostname */
+	char pb[256];  /* service name */
+	int gnie;      /* getnameinfo result code */
 
 	gnie = getnameinfo(ai->ai_addr, ai->ai_addrlen, buf, sizeof(buf), NULL, 0, NI_NUMERICHOST);
 	if (gnie)
@@ -299,14 +304,17 @@ int SockOpen(const char *host, const char *service,
 
 	if (outlevel >= O_VERBOSE)
 	    report_build(stdout, GT_("Trying to connect to %s/%s..."), buf, pb);
-	i = socket(ai->ai_family, ai->ai_socktype, 0);
+	i = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
 	if (i < 0) {
+	    int e = errno;
 	    /* mask EAFNOSUPPORT errors, they confuse users for
 	     * multihomed hosts */
 	    if (errno != EAFNOSUPPORT)
 		acterr = errno;
 	    if (outlevel >= O_VERBOSE)
-		report_complete(stdout, GT_("cannot create socket: %s\n"), strerror(errno));
+		report_complete(stdout, GT_("cannot create socket: %s\n"), strerror(e));
+	    snprintf(errbuf+strlen(errbuf), sizeof(errbuf)-strlen(errbuf),\
+		     GT_("name %d: cannot create socket family %d type %d: %s\n"), ord, ai->ai_family, ai->ai_socktype, strerror(e));
 	    continue;
 	}
 
@@ -323,8 +331,9 @@ int SockOpen(const char *host, const char *service,
 
 	    if (outlevel >= O_VERBOSE)
 		report_complete(stdout, GT_("connection failed.\n"));
-	    if (outlevel > O_SILENT)
+	    if (outlevel >= O_VERBOSE)
 		report(stderr, GT_("connection to %s:%s [%s/%s] failed: %s.\n"), host, service, buf, pb, strerror(e));
+	    snprintf(errbuf+strlen(errbuf), sizeof(errbuf)-strlen(errbuf), GT_("name %d: connection to %s:%s [%s/%s] failed: %s.\n"), ord, host, service, buf, pb, strerror(e));
 	    fm_close(i);
 	    i = -1;
 	    continue;
@@ -342,8 +351,10 @@ int SockOpen(const char *host, const char *service,
     fm_freeaddrinfo(*ai0);
     *ai0 = NULL;
 
-    if (i == -1)
+    if (i == -1) {
+	report(stderr, GT_("Connection errors for this poll:\n%s"), errbuf);
 	errno = acterr;
+    }
 
     return i;
 }
@@ -582,7 +593,6 @@ SSL *SSLGetContext( int sock )
 	return _ssl_context[sock];
 }
 
-
 /* ok_return (preverify_ok) is 1 if this stage of certificate verification
    passed, or 0 if it failed. This callback lets us display informative
    errors, and perform additional validation (e.g. CN matches) */
@@ -666,10 +676,9 @@ static int SSL_verify_callback( int ok_return, X509_STORE_CTX *ctx, int strict )
 			if (_ssl_server_cname != NULL) {
 				char *p1 = buf;
 				char *p2 = _ssl_server_cname;
-				int n;
 				int matched = 0;
 				STACK_OF(GENERAL_NAME) *gens;
-				
+
 				/* RFC 2595 section 2.4: find a matching name
 				 * first find a match among alternative names */
 				gens = (STACK_OF(GENERAL_NAME) *)X509_get_ext_d2i(x509_cert, NID_subject_alt_name, NULL, NULL);
@@ -678,39 +687,27 @@ static int SSL_verify_callback( int ok_return, X509_STORE_CTX *ctx, int strict )
 					for (j = 0, r = sk_GENERAL_NAME_num(gens); j < r; ++j) {
 						const GENERAL_NAME *gn = sk_GENERAL_NAME_value(gens, j);
 						if (gn->type == GEN_DNS) {
-							char *p1 = (char *)gn->d.ia5->data;
-							char *p2 = _ssl_server_cname;
+							char *pp1 = (char *)gn->d.ia5->data;
+							char *pp2 = _ssl_server_cname;
 							if (outlevel >= O_VERBOSE) {
-								report(stdout, GT_("Subject Alternative Name: %s\n"), (tt = sdump(p1, (size_t)gn->d.ia5->length)));
+								report(stdout, GT_("Subject Alternative Name: %s\n"), (tt = sdump(pp1, (size_t)gn->d.ia5->length)));
 								xfree(tt);
 							}
 							/* Name contains embedded NUL characters, so we complain. This
 							 * is likely a certificate spoofing attack. */
-							if ((size_t)gn->d.ia5->length != strlen(p1)) {
+							if ((size_t)gn->d.ia5->length != strlen(pp1)) {
 								report(stderr, GT_("Bad certificate: Subject Alternative Name contains NUL, aborting!\n"));
 								sk_GENERAL_NAME_free(gens);
 								return 0;
 							}
-							if (*p1 == '*') {
-								++p1;
-								n = strlen(p2) - strlen(p1);
-								if (n >= 0)
-									p2 += n;
-							}
-							if (0 == strcasecmp(p1, p2)) {
-								matched = 1;
+							if (name_match(pp1, pp2)) {
+							    matched = 1;
 							}
 						}
 					}
 					sk_GENERAL_NAME_free(gens);
 				}
-				if (*p1 == '*') {
-					++p1;
-					n = strlen(p2) - strlen(p1);
-					if (n >= 0)
-						p2 += n;
-				}
-				if (0 == strcasecmp(p1, p2)) {
+				if (name_match(p1, p2)) {
 					matched = 1;
 				}
 				if (!matched) {
@@ -1009,8 +1006,8 @@ int SSLOpen(int sock, char *mycert, char *mykey, const char *myproto, int certck
 		}
 	}
 
-	if (!certck && (SSL_get_verify_result(_ssl_context[sock]) != X509_V_OK
-|| !_verify_ok)) {
+	if (!certck && !fingerprint &&
+		(SSL_get_verify_result(_ssl_context[sock]) != X509_V_OK || !_verify_ok)) {
 		report(stderr, GT_("Warning: the connection is insecure, continuing anyways. (Better use --sslcertck!)\n"));
 	}
 
